==> db4o-5.0-mono.tar.gz: db4o-5.0/src/test/src/com/db4o/test/SelectDistinct.cs <==
/* Copyright (C) 2004 - 2005  db4objects Inc.  http://www.db4o.com

This file is part of the db4o open source object database.

db4o is free software; you can redistribute it and/or modify it under
the terms of version 2 of the GNU General Public License as published
by the Free Software Foundation and as clarified by db4objects' GPL 
interpretation policy, available at
http://www.db4o.com/about/company/legalpolicies/gplinterpretation/
Alternatively you can write to db4objects, Inc., 1900 S Norfolk Street,
Suite 350, San Mateo, CA 94403, USA.

db4o is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */
using System;
using System.Collections;
using com.db4o.query;

namespace com.db4o.test {

    public class SelectDistinct {
        public String name;

        public SelectDistinct() {
        }

        public SelectDistinct(String name) {
            this.name = name;
        }

        public void store() {
            Tester.store(new SelectDistinct("a"));
            Tester.store(new SelectDistinct("a"));
            Tester.store(new SelectDistinct("a"));
            Tester.store(new SelectDistinct("b"));
            Tester.store(new SelectDistinct("b"));
            Tester.store(new SelectDistinct("c"));
            Tester.store(new SelectDistinct("c"));
            Tester.store(new SelectDistinct("d"));
            Tester.store(new SelectDistinct("e"));
        }

        public void test() {
            
            String[] expected = new String[]{"a", "b", "c", "d", "e"};
            
            Query q = Tester.query();
            q.constrain(typeof(SelectDistinct));
            q.constrain(new DistinctEvaluation());
            
            ObjectSet objectSet = q.execute();
            while(objectSet.hasNext()) {
                SelectDistinct sd = (SelectDistinct)objectSet.next();
                bool found = false;
                for(int i = 0; i < expected.Length; i++) {
                    if(sd.name.Equals(expected[i])) {
                        expected[i] = null;
                        found = true;
                        break;
                    }
                }
                Tester.ensure(found);
            }

            for(int i = 0; i < expected.Length; i++) {
                Tester.ensure(expected[i] == null);
            }
        }

        public class DistinctEvaluation: Evaluation{

            private Hashtable ht = new Hashtable();

            public void evaluate(Candidate candidate){
                SelectDistinct sd = (SelectDistinct)candidate.getObject();
                bool isDistinct = ht[sd.name] == null;
                candidate.include(isDistinct);
                if(isDistinct){
                    ht[sd.name] = new Object();
                }
            }
        }
    }
}

==> db4o-5.5-mono.tar.gz: db4o-5.5/src/test/src/com/db4o/test/SelectDistinct.cs <==
/* Copyright (C) 2004 - 2006  db4objects Inc.  http://www.db4o.com

This file is part of the db4o open source object database.

db4o is free software; you can redistribute it and/or modify it under
the terms of version 2 of the GNU General Public License as published
by the Free Software Foundation and as clarified by db4objects' GPL 
interpretation policy, available at
http://www.db4o.com/about/company/legalpolicies/gplinterpretation/
Alternatively you can write to db4objects, Inc., 1900 S Norfolk Street,
Suite 350, San Mateo, CA 94403, USA.

db4o is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */
using System;
using System.Collections;
using com.db4o.query;

namespace com.db4o.test {

    public class SelectDistinct {
        public String name;

        public SelectDistinct() {
        }

        public SelectDistinct(String name) {
            this.name = name;
        }

        public void Store() {
            Tester.Store(new SelectDistinct("a"));
            Tester.Store(new SelectDistinct("a"));
            Tester.Store(new SelectDistinct("a"));
            Tester.Store(new SelectDistinct("b"));
            Tester.Store(new SelectDistinct("b"));
            Tester.Store(new SelectDistinct("c"));
            Tester.Store(new SelectDistinct("c"));
            Tester.Store(new SelectDistinct("d"));
            Tester.Store(new SelectDistinct("e"));
        }

        public void Test() {
            
            String[] expected = new String[]{"a", "b", "c", "d", "e"};
            
            Query q = Tester.Query();
            q.Constrain(typeof(SelectDistinct));
            q.Constrain(new DistinctEvaluation());
            
            ObjectSet objectSet = q.Execute();
            while(objectSet.HasNext()) {
                SelectDistinct sd = (SelectDistinct)objectSet.Next();
                bool found = false;
                for(int i = 0; i < expected.Length; i++) {
                    if(sd.name.Equals(expected[i])) {
                        expected[i] = null;
                        found = true;
                        break;
                    }
                }
                Tester.Ensure(found);
            }

            for(int i = 0; i < expected.Length; i++) {
                Tester.Ensure(expected[i] == null);
            }
        }

        public class DistinctEvaluation: Evaluation{

            private Hashtable ht = new Hashtable();

            public void Evaluate(Candidate candidate){
                SelectDistinct sd = (SelectDistinct)candidate.GetObject();
                bool isDistinct = ht[sd.name] == null;
                candidate.Include(isDistinct);
                if(isDistinct){
                    ht[sd.name] = new Object();
                }
            }
        }
    }
}
